#include <libporty/net_async/abgproc.h>
#include <libporty/net/local.h>
#include "sockev.h"
#include "sockev_internal.h"

#include <stdio.h>

int sockev_abgproc(sockev_t *id_io, char *argv[], sockev_t *id_ctrl, sockev_t *id_err)
{
	int ret, child_fd[3];
	P_net_socket ctrlfd, parent[3], child[3], fdlen;

	/* prepare stdin/stdout */
	if (P_net_local_socket(&parent[0], &child[0], P_net_nonblock_full) < 0)
		return -1;

/*	printf("id_io/0=%d child[0]=%d parent[0]=%d\n", *id_io, child[0], parent[0]);
	fflush(stdout);*/

	if (P_net_local_socket(&parent[1], &child[1], P_net_nonblock_full) < 0) {
		P_net_close(parent[0]);
		P_net_close(child[0]);
		return -1;
	}

/*	printf("id_io/1=%d child[1]=%d parent[1]=%d\n", *id_io, child[1], parent[1]);
	fflush(stdout);*/

	child_fd[0] = 0;
	child_fd[1] = 1;

	if (id_err != NULL) {
		if (P_net_local_socket(&parent[2], &child[2], P_net_nonblock_full) < 0) {
			P_net_close(parent[0]);
			P_net_close(child[0]);
			P_net_close(parent[1]);
			P_net_close(child[1]);
			return -1;
		}
		
		child_fd[2] = 2;
		fdlen = 3;
	}
	else
		fdlen = 2;

/*	printf("id_ctrl=%d child[2]=%d parent[2]=%d\n", *id_ctrl, child[2], parent[2]);
	fflush(stdout);*/


#ifdef SOCKEV_ABGPROC_DEBUG
	{
		int n;
		printf("Inherit:\n");
		for(n = 0; n < fdlen; n++)
			printf("  parent=%d child=%d child_fd=%d\n", parent[n], child[n], child_fd[n]);
	}
#endif

	ret = P_net_async_bgproc_inherit(argv, (id_ctrl == NULL) ? NULL : &ctrlfd, child, child_fd, fdlen);

	if (ret != 0) {
		P_net_close(parent[0]);
		P_net_close(child[0]);
		P_net_close(parent[1]);
		P_net_close(child[1]);
		P_net_close(parent[2]);
		P_net_close(child[2]);
		return -1;
	}

	if (*id_io == SOCKEV_NEW)
		*id_io = sockev_alloc();
	if (*id_ctrl == SOCKEV_NEW)
		*id_ctrl = sockev_alloc();
	if ((id_err != NULL) && (*id_err == SOCKEV_NEW))
		*id_err = sockev_alloc();

	SOCKS[*id_io].s = parent[1];
	SOCKS[*id_io].sw = parent[0];
	sockev_setfds(*id_io, parent[1], parent[0]);
	SOCKS[*id_io].state = ST_COMM;
	SOCKS[*id_io].alive = SA_WR | SA_RD;
	SOCKS[*id_io].watch = SA_WR | SA_RD;
	sockev_setpoll(*id_io);
	if (SOCKS[*id_io].hooks.open != NULL)
		SOCKS[*id_io].hooks.open(*id_io);


	if (*id_ctrl != SOCKEV_INVALID) {
		SOCKS[*id_ctrl].s = ctrlfd;
		sockev_setfds(*id_ctrl, ctrlfd, P_NET_INVALID_SOCKET);
		SOCKS[*id_ctrl].state = ST_COMM;
		SOCKS[*id_ctrl].alive = SA_WR | SA_RD;
		SOCKS[*id_ctrl].watch = SA_WR | SA_RD;
		sockev_setpoll(*id_ctrl);
	}
	if (SOCKS[*id_ctrl].hooks.open != NULL)
		SOCKS[*id_ctrl].hooks.open(*id_ctrl);

	if (id_err != NULL) {
		if (*id_err != SOCKEV_INVALID) {
			SOCKS[*id_err].s = parent[2];
			PFDS[(*id_err)*2].fd = parent[2];
			sockev_setfds(*id_err, parent[2], P_NET_INVALID_SOCKET);
			SOCKS[*id_err].state = ST_COMM;
			SOCKS[*id_err].alive = SA_RD;
			SOCKS[*id_err].watch = SA_RD;
			sockev_setpoll(*id_err);
		}
		if (SOCKS[*id_err].hooks.open != NULL)
			SOCKS[*id_err].hooks.open(*id_err);
	}

	/* no need to keep child fds open on parent side */
	P_net_close(child[0]);
	P_net_close(child[1]);
	P_net_close(child[2]);

	return 0;
}




