#include <libporty/net/tcp4.h>
#include "sockev.h"
#include "sockev_internal.h"

sockev_t sockev_tcp4_connectb(sockev_t id, const char *hostname, int port, const char *lhostname, int lport)
{
	P_net_socket S;

	if (P_tcp4_connect(&S, hostname, port, lhostname, lport, P_net_nonblock_full) < 0)
		return SOCKEV_INVALID;

	if (id == SOCKEV_NEW)
		id = sockev_alloc();
	SOCKS[id].s = S;
	SOCKS[id].state = ST_CONNECT;
	SOCKS[id].alive = SA_WR; /* allow write - it's buffered */
	SOCKS[id].watch = SA_WR; /* poll for write */
	sockev_setfds(id, S, P_NET_INVALID_SOCKET);
	sockev_setpoll(id);
	return id;
}


sockev_t sockev_tcp4_connect(sockev_t id, const char *hostname, int port)
{
	return sockev_tcp4_connectb(id, hostname, port, NULL, 0);
}

sockev_t sockev_tcp4_listen(sockev_t id, const char *hostname, int port, int backlog)
{
	P_net_socket S;

	if (P_tcp4_listen(&S, hostname, port, P_net_nonblock_full, backlog) < 0)
		return SOCKEV_INVALID;

	if (id == SOCKEV_NEW)
		id = sockev_alloc();
	SOCKS[id].s = S;
	SOCKS[id].state = ST_LISTEN;
	SOCKS[id].alive = SA_RD;
	SOCKS[id].watch = SA_RD;
	sockev_setfds(id, S, P_NET_INVALID_SOCKET);
	sockev_setpoll(id);
	if (SOCKS[id].hooks.open != NULL)
		SOCKS[id].hooks.open(id);

	return id;
}

sockev_t sockev_tcp4_accept(sockev_t id, sockev_t listener_sockev)
{
	P_net_socket S;
	P_ipv4_addr_t rem_ip;
	int rem_port;

	if (wrong_id(listener_sockev, ST_LISTEN))
		return -1;

	if (P_tcp4_accept(&S, SOCKS[listener_sockev].s, &rem_ip, NULL, &rem_port, P_net_nonblock_full) < 0)
		return SOCKEV_INVALID;

#warning async resolve hostname

	if (id == SOCKEV_NEW)
		id = sockev_alloc();
	SOCKS[id].s = S;
	SOCKS[id].state = ST_COMM;
	SOCKS[id].remote_ip = rem_ip;
	SOCKS[id].remote_host = NULL;
	SOCKS[id].remote_port = rem_port;
	SOCKS[id].alive = SA_RD | SA_WR;
	SOCKS[id].watch = SA_RD | SA_WR;
	sockev_setfds(id, S, P_NET_INVALID_SOCKET);
	sockev_setpoll(id);

	if (SOCKS[id].hooks.open != NULL)
		SOCKS[id].hooks.open(id);

	return id;
}
